from django.conf import settings
from django.utils.translation import ugettext_lazy as _

# markdown.markdown kwargs
MARKDOWNX_MARKDOWN_KWARGS = getattr(settings, 'MARKDOWNX_MARKDOWN_KWARGS', dict())

# path
MARKDOWNX_MEDIA_PATH = getattr(settings, 'MARKDOWNX_MEDIA_PATH', 'markdownx/')

# image
MARKDOWNX_MAX_UPLOAD_SIZE = getattr(settings, 'MARKDOWNX_MAX_UPLOAD_SIZE', 52428800) # 50MB
MARKDOWNX_CONTENT_TYPES = getattr(settings, 'MARKDOWNX_CONTENT_TYPES', ['image/jpeg', 'image/png'])
MARKDOWNX_IMAGE_SIZE = getattr(settings, 'MARKDOWNX_IMAGE_SIZE', {'size': (500, 500), 'quality': 90,})

# translations
LANGUAGES = getattr(settings, 'LANGUAGES', (
    ('en', _('English')),
    ('pl', _('Polish')),
)
)



from django.conf.urls import url

from .views import (
    ImageUploadView,
    MarkdownifyView,
)

urlpatterns = [
    url(r'^upload/$', ImageUploadView.as_view()),
    url(r'^markdownify/$', MarkdownifyView.as_view()),
]



import os.path
import uuid
import StringIO

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.template import defaultfilters as filters

from .utils import scale_and_crop
from .settings import (
    MARKDOWNX_IMAGE_SIZE,
    MARKDOWNX_MEDIA_PATH,
    MARKDOWNX_CONTENT_TYPES,
    MARKDOWNX_MAX_UPLOAD_SIZE,
)

class ImageForm(forms.Form):
    image = forms.ImageField()

    def save(self, commit=True):
        img = scale_and_crop(self.files['image'], **MARKDOWNX_IMAGE_SIZE)
        thumb_io = StringIO.StringIO()
        img.save(thumb_io,  self.files['image'].content_type.split('/')[-1].upper())
        
        file_name = str(self.files['image'])
        img = InMemoryUploadedFile(thumb_io, "image", file_name, self.files['image'].content_type, thumb_io.len, None)
        
        unique_file_name = self.get_unique_file_name(file_name)
        full_path = settings.MEDIA_ROOT + MARKDOWNX_MEDIA_PATH + unique_file_name
        if not os.path.exists(os.path.dirname(full_path)):
            os.makedirs(os.path.dirname(full_path))
        
        destination = open(full_path, 'wb+')
        for chunk in img.chunks():
            destination.write(chunk)
        destination.close()

        return settings.MEDIA_URL + MARKDOWNX_MEDIA_PATH + unique_file_name

    def get_unique_file_name(instance, filename):
        ext = filename.split('.')[-1]
        filename = "%s.%s" % (uuid.uuid4(), ext)
        return filename

    def clean(self):
        upload = self.cleaned_data['image']
        content_type = upload.content_type
        if content_type in MARKDOWNX_CONTENT_TYPES:
            if upload._size > MARKDOWNX_MAX_UPLOAD_SIZE:
                raise forms.ValidationError(_('Please keep filesize under %(max)s. Current filesize %(current)s') % {'max':filters.filesizeformat(MARKDOWNX_MAX_UPLOAD_SIZE), 'current':filters.filesizeformat(upload._size)})
        else:
            raise forms.ValidationError(_('File type is not supported'))

        return upload



from django.views.generic.edit import View, FormView
from django.http import HttpResponse, JsonResponse

import markdown

from . import forms
from .settings import MARKDOWNX_MARKDOWN_KWARGS


class MarkdownifyView(View):

    def post(self, request, *args, **kwargs):
        return HttpResponse(markdown.markdown(request.POST['content'], **MARKDOWNX_MARKDOWN_KWARGS))


class ImageUploadView(FormView):
    template_name = "dummy.html"
    form_class = forms.ImageForm
    success_url = '/'

    def form_invalid(self, form):
        response = super(ImageUploadView, self).form_invalid(form)
        if self.request.is_ajax():
            return JsonResponse(form.errors, status=400)
        else:
            return response
 
    def form_valid(self, form):
        image_path = form.save()
        response = super(ImageUploadView, self).form_valid(form)

        if self.request.is_ajax():
            data = {}
            data['image_path'] = image_path
            return JsonResponse(data)
        else:
            return response



from PIL import Image

def scale_and_crop(image, size, crop=False, upscale=False, quality=None):
    # Open image and store format/metadata.
    image.open()
    im = Image.open(image)
    im_format, im_info = im.format, im.info
    if quality:
        im_info['quality'] = quality

    # Force PIL to load image data.
    im.load()

    source_x, source_y = [float(v) for v in im.size]
    target_x, target_y = [float(v) for v in size]

    if crop or not target_x or not target_y:
        scale = max(target_x / source_x, target_y / source_y)
    else:
        scale = min(target_x / source_x, target_y / source_y)

    # Handle one-dimensional targets.
    if not target_x:
        target_x = source_x * scale
    elif not target_y:
        target_y = source_y * scale

    if scale < 1.0 or (scale > 1.0 and upscale):
        im = im.resize((int(source_x * scale), int(source_y * scale)),
                       resample=Image.ANTIALIAS)

    if crop:
        # Use integer values now.
        source_x, source_y = im.size
        # Difference between new image size and requested size.
        diff_x = int(source_x - min(source_x, target_x))
        diff_y = int(source_y - min(source_y, target_y))
        if diff_x or diff_y:
            # Center cropping (default).
            halfdiff_x, halfdiff_y = diff_x // 2, diff_y // 2
            box = [halfdiff_x, halfdiff_y,
                   min(source_x, int(target_x) + halfdiff_x),
                   min(source_y, int(target_y) + halfdiff_y)]
            # Finally, crop the image!
            im = im.crop(box)

    # Close image and replace format/metadata, as PIL blows this away.
    im.format, im.info = im_format, im_info
    image.close()
    return im



from django.conf import settings
from django.forms import Textarea
from django.template import Context
from django.template.loader import get_template


class MarkdownxInput(Textarea):
    def __init__(self, attrs=None):

        default_attrs = {
            'id': 'markdownx_editor',
        }
        if attrs:
            default_attrs.update(attrs)

        super(Textarea, self).__init__(default_attrs)

    def render(self, name, value, attrs=None):
        textarea = Textarea.render(self, name, value)

        t = get_template('markdownx/widget.html')
        c = Context({
            'markdownx_editor': textarea,
        })

        return t.render(c)

    class Media:
        js = (
            'js/markdownx.js',
        )



from setuptools import setup, find_packages

import os
if 'vagrant' in str(os.environ):
    del os.link

setup(
    name='django-markdownx',
    version='0.3.0',
    packages=find_packages(),
    include_package_data=True,
    description='Simple markdown editor (with live preview and images uploads) built for Django',
    url='https://github.com/adi-/django-markdownx',
    author='Adrian Drzewicki',
    author_email='adrian@enove.pl',
    license='BSD',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Topic :: Software Development',
        'Topic :: Software Development :: User Interfaces',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: JavaScript',
    ],
    keywords='django markdown live preview images upload',
    install_requires=['Pillow', 'Markdown'],
)
